/* File:   Basic4GLPluginMacros.h
 * Author: Joshy (d.j.peters)
 */

#ifndef _B4GLPLUGINMACROS_H
#define _B4GLPLUGINMACROS_H
/* registry helper */


// register int float string const's
#define RIC(name,value) registry.RegisterIntConstant(#name,value)
#define RFC(name,value) registry.RegisterFloatConstant(#name,value)
#define RSC(name,value) registry.RegisterStringConstant(#name,(const char *)value)

#define RIC2(name) registry.RegisterIntConstant(#name,name)
#define RFC2(name) registry.RegisterFloatConstant(#name,name)

// register void int float string function
#define RVF(name) registry.RegisterVoidFunction(#name,__##name)
#define RIF(name) registry.RegisterFunction(#name,__##name, DLL_BASIC4GL_INT)
#define RFF(name) registry.RegisterFunction(#name,__##name, DLL_BASIC4GL_FLOAT)
#define RSF(name) registry.RegisterFunction(#name,__##name, DLL_BASIC4GL_STRING)

// register int,float,string array function by dimension
#define RIAF(name,d) registry.RegisterArrayFunction(#name,__##name,DLL_BASIC4GL_INT,d)
#define RFAF(name,d) registry.RegisterArrayFunction(#name,__##name,DLL_BASIC4GL_FLOAT,d)
#define RSAF(name,d) registry.RegisterArrayFunction(#name,__##name,DLL_BASIC4GL_STRING,d)
  
// add int float string param
#define AIP registry.AddParam(DLL_BASIC4GL_INT)
#define AFP registry.AddParam(DLL_BASIC4GL_FLOAT)
#define ASP registry.AddParam(DLL_BASIC4GL_STRING)

// register int,float,string,struct param as reference
#define AIPR registry.AddParam(DLL_BASIC4GL_INT);registry.ModParamReference()
#define AFPR registry.AddParam(DLL_BASIC4GL_FLOAT);registry.ModParamReference()
#define ASPR registry.AddParam(DLL_BASIC4GL_STRING);registry.ModParamReference()
#define ATPR(h) registry.AddStrucParam(_registry,h);registry.ModParamReference()

// register pointer on int,float,string param
#define AIPP registry.AddParam(DLL_BASIC4GL_INT);registry.ModParamPointer(1)
#define AFPP registry.AddParam(DLL_BASIC4GL_FLOAT);registry.ModParamPointer(1)
#define ASPP registry.AddParam(DLL_BASIC4GL_STRING);registry.ModParamPointer(1)
#define ATPP(h) registry.AddStrucParam(_registry,h);registry.ModParamPointer(1)
  
// add int,float,string array param by dimensions
#define AIAP(d) registry.AddArrayParam(DLL_BASIC4GL_INT   ,d)
#define AFAP(d) registry.AddArrayParam(DLL_BASIC4GL_FLOAT ,d)
#define ASAP(d) registry.AddArrayParam(DLL_BASIC4GL_STRING,d)


/* runtime helper */

// declare plugin function
#define DPF(name) void DLLFUNC __##name(IDLL_Basic4GL_Runtime &basic4gl)

// get int float string parameter
#define GI(n) basic4gl.GetIntParam(n)
#define GF(n) basic4gl.GetFloatParam(n)
#define GS(n) basic4gl.GetStringParam(n)

// get array size 1D
#define GAD(index) basic4gl.GetArrayParamDimension(index,0)
// get array size 2D
#define GAD2(index) basic4gl.GetArrayParamDimension(index,1)

// get int float array param array()
#define GIA(n,pArray,n1) basic4gl.GetIntArrayParam(n,pArray,1,n1)
#define GFA(n,pArray,n1) basic4gl.GetFloatArrayParam(n,pArray,1,n1)
// 2 dimensions array()()
#define GIA2(n,pArray,n1,n2) basic4gl.GetIntArrayParam(n,pArray,2,n1,n2)
#define GFA2(n,pArray,n1,n2) basic4gl.GetFloatArrayParam(n,pArray,2,n1,n2)

// set byte,word,int,float,string param (byref)
#define SB(n,v) basic4gl.SetType(DLL_BASIC4GL_EXT_BYTE);basic4gl.ModTypeReference(); basic4gl.SetParam(n,v)
#define SW(n,v) basic4gl.SetType(DLL_BASIC4GL_EXT_WORD);basic4gl.ModTypeReference(); basic4gl.SetParam(n,v)
#define SI(n,v) basic4gl.SetType(DLL_BASIC4GL_EXT_INT);basic4gl.ModTypeReference(); basic4gl.SetParam(n,v)
#define SF(n,v) basic4gl.SetType(DLL_BASIC4GL_EXT_FLOAT);basic4gl.ModTypeReference(); basic4gl.SetParam(n,v)
#define SS(n,s,v) basic4gl.SetStringType(s);basic4gl.ModTypeReference();basic4gl.SetParam(n,v)
// set int,float array() param byref 
#define SIA(n,p,n1) basic4gl.SetType(DLL_BASIC4GL_EXT_INT);basic4gl.ModTypeArray(1,n1);basic4gl.SetParam(n,p);
#define SFA(n,p,n1) basic4gl.SetType(DLL_BASIC4GL_EXT_FLOAT);basic4gl.ModTypeArray(1,n1);basic4gl.SetParam(n,p);
// 2 dimension array()()= ...
#define SIA2(n,p,n1,n2) basic4gl.SetType(DLL_BASIC4GL_EXT_INT);basic4gl.ModTypeArray(2,n1,n2);basic4gl.SetParam(n,p);
#define SFA2(n,p,n1,n2) basic4gl.SetType(DLL_BASIC4GL_EXT_FLOAT);basic4gl.ModTypeArray(2,n1,n2);basic4gl.SetParam(n,p);

// set int float string result
#define SIR(r) basic4gl.SetIntResult((int) r)
#define SFR(r) basic4gl.SetFloatResult(r)
#define SSR(r) basic4gl.SetStringResult(r)

// set int float array() result
#define SIAR(p,n1) basic4gl.SetFloatArrayResult(p,1,n1);
#define SFAR(p,n1) basic4gl.SetFloatArrayResult(p,1,n1);
// set int float array()() result
#define SIAR2(p,n1,n2) basic4gl.SetFloatArrayResult(p,2,n1,n2);
#define SFAR2(p,n1,n2) basic4gl.SetFloatArrayResult(p,2,n1,n2);
// direct set int float
#define DSI(p,v) basic4gl.DirectSetInt(p,(int) v)
#define DSF(p,v) basic4gl.DirectSetFloat(p,v)

#endif /* _B4GLPLUGINMACROS_H */

