// FontPlugin.cpp by DJLinux (Joshy)

#include <malloc.h>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <GL/GL.h>

#include <Basic4GLDLLInterface.h>
#include <Basic4GLStandardObjects.h>
#include "Basic4GLPluginMacros.h"

///////////////////////////////////////////////////////////////////////////////
// FontPlugin
//

// Global variables.

char  * pluginError = NULL;
IB4GLCompiler * gpCompiler  = NULL;

class FontPlugin : public IDLL_Basic4GL_Plugin {
public:
FontPlugin() { ; }
virtual bool DLLFUNC Load(IDLL_Basic4GL_FunctionRegistry &registry, bool isStandaloneExe);
virtual bool DLLFUNC Start();
virtual void DLLFUNC End();
virtual void DLLFUNC Unload();
virtual void DLLFUNC GetError(char *error);
virtual void DLLFUNC Pause();
virtual void DLLFUNC Resume();
virtual void DLLFUNC DelayedResume();
virtual void DLLFUNC ProcessMessages();
};

typedef void (WINAPI *WINPOS2I)(int x, int y);
typedef void (WINAPI *WINPOS2IV)(int * array);
typedef void (WINAPI *WINPOS2F)(float x, float y);
typedef void (WINAPI *WINPOS2FV)(float * array);

typedef void (WINAPI *WINPOS3I)(int x, int y, int z);
typedef void (WINAPI *WINPOS3IV)(int * array);
typedef void (WINAPI *WINPOS3F)(float x, float y, float z);
typedef void (WINAPI *WINPOS3FV)(float * array);




#define GETARRAY(index,type) \
  type n=GAD(index); \
  type * a = new type[n]; \
  GIA(index,a,n);

#define DELARRAY delete[] a;

HWND  mWin=NULL;
HDC   mDC =NULL;;
HGLRC mRC =NULL;
PIXELFORMATDESCRIPTOR mPFD;


WINPOS2I  glWindowPos2i;
WINPOS2IV glWindowPos2iv;
WINPOS3I  glWindowPos3i;
WINPOS3IV glWindowPos3iv;

WINPOS2F  glWindowPos2f;
WINPOS2FV glWindowPos2fv;
WINPOS3F  glWindowPos3f;
WINPOS3FV glWindowPos3fv;


int mFontBase=0;
void * mOldFont=NULL;

int mHeight;
int mWidth;
int mEscapement;
int mOrientation;
int mWeight;
int mItalic;
int mUnderline;
int mStrikeout;
int mCharset;
int mOutPrecision;
int mClipPrecision;
int mQuality;
int mPitchandFamily;

int nFonts;
#define MAX_FONTS 512
char ** pFontNames=0;

int CALLBACK EnumFontFamProc(LPENUMLOGFONT lpelf, LPNEWTEXTMETRIC lpntm, DWORD nFontType, long lParam) {
  if (nFonts<MAX_FONTS) {
    lpelf->elfLogFont.lfFaceName;
    char * pFaceName = (char *)lpelf->elfLogFont.lfFaceName;
    int nChars;
    for (nChars=0;pFaceName[nChars];nChars++);
    if (nChars>0) {
      pFontNames[nFonts]=(char*)malloc(nChars+1);
      char * pFontName=pFontNames[nFonts];
      for (int i=0;i<nChars;i++) {
        pFontName[i]=pFaceName[i];
      }
      pFontName[nChars]=0;
      nFonts++;
    }
    return 1;
  }
  return 0;
} 

void AllocFontNames(void){
  if (!pFontNames)
    pFontNames = (char**)malloc(sizeof(char*)*MAX_FONTS);

  LOGFONT lLogFont;
  lLogFont.lfCharSet = DEFAULT_CHARSET;
  lLogFont.lfFaceName[0] = NULL;
  lLogFont.lfPitchAndFamily = 0;
  EnumFontFamiliesEx(mDC, &lLogFont,(FONTENUMPROC) EnumFontFamProc, 0, 0); 
}

void FreeFontNames(void){
  if (pFontNames) {
    if (nFonts>0) {
      for (int i=0;i<MAX_FONTS;i++){
        if (pFontNames[i]) {
          free((void*)pFontNames[i]);
          pFontNames[i]=0;
        }
      }
      nFonts=0;
    }   
    free(pFontNames);
    pFontNames=0;
  }
}

///////////////////////////////////////////////////////////////////////////////
// Runtime functions
// These are the functions that Basic4GL calls

DPF(GetFontCount){ //nFonts%=()
  SIR(nFonts);
}

DPF(GetFontName){ // name$=(index)
  int index=GI(1);
  if ((index>-1) && (index<nFonts)) {
    SSR(pFontNames[index]);
  }
}

//NewFont(face$)
DPF(NewFont) { 
  int h= mHeight;
  int w= mWidth;
  if (h!=0)
    h=-MulDiv(h, GetDeviceCaps(GetDC(0), LOGPIXELSY), 72);
  if (w!=0)
    w=-MulDiv(w, GetDeviceCaps(GetDC(0), LOGPIXELSY), 72);
  SIR(CreateFont(h,w, mEscapement, mOrientation, mWeight, mItalic, mUnderline, mStrikeout,  mCharset, mOutPrecision,mClipPrecision, mQuality, mPitchandFamily,GS(1)) );
}

DPF(CreateFont) {
  int h= GI(14);
  int w= GI(13);
  if (h!=0)
    h=-MulDiv(h, GetDeviceCaps(GetDC(0), LOGPIXELSY), 72);
  if (w!=0)
    w=-MulDiv(w, GetDeviceCaps(GetDC(0), LOGPIXELSY), 72);

  SIR(CreateFont(h,w,  GI(12),         GI(11),     GI(10),  GI(9),    GI(8),       GI(7),       GI(6),      GI(5),         GI(4),      GI(3),        GI(2),   GS(1)) );
}

DPF(DeleteFont) {
  int zero=0;
  SIR( DeleteObject((void *) GI(1)) );
  SI(1,&zero);
}

DPF(SetFontHeight)        {mHeight        =GI(1);}
DPF(SetFontWidth)         {mWidth         =GI(1);}
DPF(SetFontEscapement)    {mEscapement    =GI(1);}
DPF(SetFontOrientation)   {mOrientation   =GI(1);}
DPF(SetFontWeight)        {mWeight        =GI(1);}
DPF(SetFontItalic)        {mItalic        =GI(1);}
DPF(SetFontUnderline)     {mUnderline     =GI(1);}
DPF(SetFontStrikeout)     {mStrikeout     =GI(1);}
DPF(SetFontCharset)       {mCharset       =GI(1);}
DPF(SetFontOutPrecision)  {mOutPrecision  =GI(1);}
DPF(SetFontClipPrecision) {mClipPrecision =GI(1);}
DPF(SetFontQuality)       {mQuality       =GI(1);}
DPF(SetFontPitchAndFamily){mPitchandFamily=GI(1);}

DPF(UseFont) {
  void *  hFont=(void *) GI(2);
  if (hFont==NULL)
    return;

  
  if (mOldFont!=0) {
    SelectObject(mDC,mOldFont);
    mOldFont=0;
  }
  mOldFont  = SelectObject(mDC,  hFont);
  wglUseFontBitmaps(mDC, 0, 255, GI(1));
}


DPF(glFontPos2i){
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();

  (*glWindowPos2i)(GI(2),GI(1));
  
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();
}

DPF(glWindowPos2i){
  (*glWindowPos2i)(GI(2),GI(1));
}

DPF(glWindowPos3i){
  (*glWindowPos3i)(GI(3),GI(2),GI(1));
}
DPF(glWindowPos2f){
  (*glWindowPos2f)(GF(2),GF(1));
}

DPF(glWindowPos3f){
  (*glWindowPos3f)(GF(3),GF(2),GF(1));
}

///////////////////////////////////////////////////////////////////////////////
//	FontPlugin

bool DLLFUNC FontPlugin::Load(IDLL_Basic4GL_FunctionRegistry &registry, bool isStandaloneExe) {

  // Initialise DLL stuff

  // Register shared interfaces
  /*
  gpCompiler=(IB4GLCompiler*)registry.FetchInterface("IB4GLCompiler",
  IB4GLCOMPILER_MAJOR,
  IB4GLCOMPILER_MINOR);
*/
  // Register constants

  // CreateFont fnWeight-Konstanten
  RIC(WEIGHT_DONTCARE  ,  0); // Standard
  RIC(WEIGHT_THIN      ,100); // Super Dnn
  RIC(WEIGHT_EXTRALIGHT,200); // Extra Dnn
  RIC(WEIGHT_LIGHT     ,300); // Dnn
  RIC(WEIGHT_NORMAL    ,400); // Normal
  RIC(WEIGHT_MEDIUM    ,500); // Mittel
  RIC(WEIGHT_SEMIBOLD  ,600); // Etwas Dicker
  RIC(WEIGHT_BOLD      ,700); // Fett
  RIC(WEIGHT_EXTRABOLD ,800); // Extra Fett
  RIC(WEIGHT_HEAVY     ,900); // Super Fett
  // CreateFont fdwCharSet-Konstanten
  RIC(CHARSET_ANSI       ,  0); // Ansi Zeichensatz
  RIC(CHARSET_DEFAULT    ,  1); // Standard
  RIC(CHARSET_SYMBOL     ,  2); // Symbol
  RIC(CHARSET_MAC        , 77); // Mac (Win 9x)
  RIC(CHARSET_SHIFTJIS   ,128); // ShiftJis
  RIC(CHARSET_HANGEUL    ,129); // Hangeul
  RIC(CHARSET_JOHAB      ,130); // Johab (Win 9x)
  RIC(CHARSET_GB2312     ,134); // Englisch
  RIC(CHARSET_CHINESEBIG5,136); // Chinesisch
  RIC(CHARSET_GREEK      ,161); // Griechisch (Win 9x)
  RIC(CHARSET_TURKISH    ,162); // Trkisch (Win 9x)
  RIC(CHARSET_HEBREW     ,177); // Hebrisch (NT/2000)
  RIC(CHARSET_ARABIC     ,178); // Arabisch (NT/2000)
  RIC(CHARSET_BALTIC     ,186); // Baltisch (Win 9x)
  RIC(CHARSET_RUSSIAN    ,204); // Russisch (Win 9x)
  RIC(CHARSET_THAI       ,222); // Thailndisch (NT/2000)
  RIC(CHARSET_EASTEUROPE ,238); // Osteuropisch (Win 9x)
  RIC(CHARSET_OEM        ,255); // OEM

  // CreateFont fdwClipPrecision-Konstanten
  RIC(CLIP_DEFAULT  ,  0); // Standard
  RIC(CLIP_STROKE   ,  2); // Raster Vector oder TrueType Fonts fr die Enumeration 
  RIC(CLIP_LH_ANGLES, 16); // Die Richtung von irgendwelchen Rotationen wird  vom Koordinatensystem festgelegt (gegen den Uhrzeigersinn)
  RIC(CLIP_EMBEDDED ,128); // wird benutzt fr eingebettete schreibgeschtzte Fonts 

  // CreateFont fdwOutPrecision-Konstanten
  RIC(OUT_DEFAULT, 0); // Benutzt den Standard-Font
  RIC(OUT_STRING,  1); // Raster-Font fr die Enumeration
  RIC(OUT_STROKE,  3); // Vector Font fr die Enumeration (NT/2000) TrueType, Outline-Based oder VectorFont fr die Enumeration
  RIC(OUT_TT,      4); // Benutzt ein TrueType-Font wenn andere Fonts mit dem selben Namen existieren
  RIC(OUT_DEVICE,  5); // Benutzt einen Device-Font wenn mehrere Dateien mit dem selben Namen existieren
  RIC(OUT_RASTER,  6); // Benutzt einen Raster-Font wenn mehrere Dateien mit dem selben Namen existieren#
  RIC(OUT_TT_ONLY, 7); // Benutzt eine TrueType-Schriftart
  RIC(OUT_OUTLINE, 8); // Benutzt eine TrueType-Schriftart oder Outline-Based Fonts.
  
  // CreateFont fdwQuality-Konstanten
  RIC(QUALITY_DEFAULT       , 0); // Standard Qualitt
  RIC(QUALITY_DRAFT         , 1); // Das Erscheinen des Fonts ist weniger wichtig als bei PROOF_QUALITY
  RIC(QUALITY_PROOF         , 2); // Die Qualitt der Erscheinung ist wichtiger als  das exakte bereinstimmen mit den Font-Attributen
  RIC(QUALITY_NONANTIALIASED, 3); // Die Schriftart wird nicht mit Anitaliasing gezeichnet
  RIC(QUALITY_ANTIALIASED   , 4); // Die Schriftart wird mit Anitaliasing gezeichnet

  // CreateFont fdwPitchAndFamily-Konstanten
  RIC(PITCH_DEFAULT     , 0); // Standard
  RIC(PITCH_FIXED       , 1); // Fest
  RIC(PITCH_VARIABLE    , 2); // Variabel
  
  RIC(FAMILY_DONTCARE   , 0); // Egal
  RIC(FAMILY_ROMAN      ,16); // Roman
  RIC(FAMILY_SWISS      ,32); // Swiss
  RIC(FAMILY_MODERN     ,48); // Modern
  RIC(FAMILY_SCRIPT     ,64); // Script
  RIC(FAMILY_DECORATIVE ,80); // Dekoriert

  RIF(GetFontCount); //nFonts%=()
  RSF(GetFontName);AIP; // name$=(index)

  // CreateFont%(Height%,Width%,Escapement%,Orientation%,Weight%,Italic%,Underline%,StrikeOut%,CharSet%,OutPrecision%,ClipPrecision%,Quality%,PitchAndFamily%,Face$)
  RIF(NewFont);ASP;
  RIF(CreateFont);AIP;AIP; AIP;AIP; AIP;AIP;AIP;AIP; AIP;AIP;AIP;AIP;AIP,ASP;
  RIF(DeleteFont);AIPR;
  RVF(SetFontHeight);AIP;
  RVF(SetFontWidth);AIP;
  RVF(SetFontEscapement);AIP;
  RVF(SetFontOrientation);AIP;
  RVF(SetFontWeight);AIP;
  RVF(SetFontItalic);AIP;
  RVF(SetFontUnderline);AIP;
  RVF(SetFontStrikeout);AIP;
  RVF(SetFontCharset);AIP;
  RVF(SetFontOutPrecision);AIP;
  RVF(SetFontClipPrecision);AIP;
  RVF(SetFontQuality);AIP;
  RVF(SetFontPitchAndFamily);AIP;
  // UseFont(hFont,List)
  RVF(UseFont);AIP;AIP;

  RVF(glFontPos2i);AIP;AIP;


  RVF(glWindowPos2i);AIP;AIP;
  RVF(glWindowPos3i);AIP;AIP;AIP;
  RVF(glWindowPos2f);AFP;AFP;
  RVF(glWindowPos3f);AFP;AFP;AFP;

  glWindowPos2i  = (WINPOS2I)  wglGetProcAddress("glWindowPos2i");
  glWindowPos2f  = (WINPOS2F)  wglGetProcAddress("glWindowPos2f");

  glWindowPos2iv = (WINPOS2IV) wglGetProcAddress("glWindowPos2iv");
  glWindowPos2fv = (WINPOS2FV) wglGetProcAddress("glWindowPos2fv");

  glWindowPos3i  = (WINPOS3I)  wglGetProcAddress("glWindowPos3i");
  glWindowPos3f  = (WINPOS3F)  wglGetProcAddress("glWindowPos3f");

  glWindowPos3iv = (WINPOS3IV) wglGetProcAddress("glWindowPos3iv");
  glWindowPos3fv = (WINPOS3FV) wglGetProcAddress("glWindowPos3fv");


  if (glWindowPos2i==NULL)
    return false;

  return true;
}




void DLLFUNC FontPlugin::Unload() {
  // DLL is about to unload.
  // Final cleanup code
  FreeFontNames();
  // Delete this object. This is safe because Basic4GL will not access it again.
  delete this;
}
bool DLLFUNC FontPlugin::Start() {
  
  // Program is about to start.
  // Any pre-start init code would go here.
  mWin = FindWindow("gbOPENGL",NULL);
  mDC  = GetDC(mWin);
  //mRC  = wglCreateContext(mDC);
  //int iPixelFormat = GetPixelFormat(mDC);
  //DescribePixelFormat(mDC,iPixelFormat,sizeof(PIXELFORMATDESCRIPTOR),&mPFD);
  if (mDC!=NULL) {
    //wglMakeCurrent(mDC,mRC);
    
    mHeight=16;
    mWidth=0;
    mEscapement=0;
    mOrientation=0;
    mWeight=0;
    mItalic=0;
    mUnderline=0;
    mStrikeout=0;
    mCharset=0;
    mOutPrecision=0;
    mClipPrecision=0;
    mQuality=0;
    mPitchandFamily=0;
    nFonts=0;
    AllocFontNames();
    return true;
  } else 
    return false;
}
void DLLFUNC FontPlugin::End() {
  // Program is about to end
  // Any cleanup code would go here
  if (mOldFont!=0) {
    SelectObject(mDC,mOldFont);
    mOldFont=0;
  }
  FreeFontNames();
  
}

void DLLFUNC FontPlugin::Pause() {
// Called when the program is paused (i.e. during debugging).
// A full screen window might hide itself here so that the user can see the
// Basic4GL edit window (and debug their program)
}
void DLLFUNC FontPlugin::Resume() {
// Called when the program is resumed (i.e. during debugging).
// This is also called when the user "steps"
}
void DLLFUNC FontPlugin::DelayedResume() {
// Called when the program is resumed for more than a short "step".
// This is where a fullscreen window would re-show itself (if it did so in ::Resume()
// we would get flickering when the user steps through the program)
}
void DLLFUNC FontPlugin::GetError(char *error) {
  if (pluginError != NULL) {
    for (int i = 0; pluginError[i] != 0; i++){
      error[i] = pluginError[i];
    }
  }
}
void DLLFUNC FontPlugin::ProcessMessages() {
// Called periodically (in practice, quite frequently).
// This would be a good place to process windows messages etc.
}


///////////////////////////////////////////////////////////////////////////////
// DLL exported functions
extern "C" {
  /// Query function
  DLLExport int DLLFUNC Basic4GL_Query(char *details, int *major, int *minor) {
    char *detailSrc = "for Basic4GL";
    for (int i = 0; detailSrc[i] != 0; i++){
      details[i] = detailSrc[i];
    }
    *major = 1;
    *minor = 0;

  // MUST return BASIC4GL_DLL_VERSION here!
  return BASIC4GL_DLL_VERSION;
  }

  // Main initialisation function.
  // Here we will construct and return our plugin object
  DLLExport IDLL_Basic4GL_Plugin *DLLFUNC Basic4GL_Init() {
    // Construct and return our plugin object
      return new FontPlugin();
  }
}
